@@ -1,10 +1,12 @@
-from . import commands
-from .. import git, exceptions, __version__
-from textwrap import TextWrapper
 import os
-import shutil
-import click
 import re
+import shutil
+from textwrap import TextWrapper
+
+import click
+
+from .. import __version__, exceptions, git
+from . import commands
 
 
 def ensure_top_level():
@@ -285,6 +287,7 @@ def tarball():
 @entry_point.group()
 @click.pass_context
 def cache(ctx):
+    """Caching-related operations."""
     pass
 
 
@@ -387,4 +390,4 @@ def restore(ctx):
 @click.pass_context
 def update(ctx):
     ensure_top_level()
-    commands.cache_update()
\ No newline at end of file
+    commands.cache_update()
