@@ -1,13 +1,12 @@
-from . import paths, exceptions, logging
-from .subproc import get_stdout
-import subprocess
 import os
+import re
 import shutil
 from pathlib import Path
-from urllib.parse import quote
 from tempfile import TemporaryDirectory
-import re
+from urllib.parse import quote
 
+from . import exceptions, logging, paths
+from .subproc import get_stdout
 
 OVERLEAF_BLANK_PROJECT_REGEX_TEMPLATE = r"[\n\r\s]+".join(
     [
@@ -45,6 +44,7 @@ def check_for_rate_limit(code, stdout, stderr):
 
     """
     if stdout:
+        logger = logging.get_logger()
         logger.debug(stdout)
     if code != 0:
         if "Rate limit exceeded" in stderr:
@@ -119,6 +119,13 @@ def clone(project_id, path=None):
     # storing the url containing the password in .git/config
     get_stdout(["git", "init"], cwd=str(paths.user(path=path).overleaf))
 
+    # Overleaf uses a branch called 'master' by default. If the local config
+    # uses a different name, we need to change it.
+    get_stdout(
+        ["git", "branch", "-M", "master"],
+        cwd=str(paths.user(path=path).overleaf),
+    )
+
     # Pull from the repo (hide secrets)
     def callback(code, stdout, stderr):
         if stdout:
@@ -150,6 +157,7 @@ def wipe_remote(project_id):
     """
     with TemporaryDirectory() as cwd:
         get_stdout(["git", "init"], cwd=cwd)
+        get_stdout(["git", "branch", "-M", "master"], cwd=cwd)
         (
             overleaf_email,
             overleaf_password,
@@ -577,4 +585,4 @@ def pull_files(
         ],
         cwd=paths.user(path=path).repo,
         callback=callback,
-    )
\ No newline at end of file
+    )
