@@ -1,10 +1,10 @@
+import requests
+
 from . import exceptions
-from .logging import get_logger
-from .patches import patch_snakemake_cache
 from .config import get_run_type
 from .git import get_repo_branch
-import json
-import requests
+from .logging import get_logger
+from .patches import patch_snakemake_cache
 
 try:
     import snakemake
@@ -35,6 +35,12 @@ def process_user_rules():
     if cache_zenodo_doi or cache_sandbox_doi:
         patch_snakemake_cache(cache_zenodo_doi, cache_sandbox_doi)
 
+    # Remind the user to publish the deposit
+    if cache_sandbox_doi and get_run_type == "build":
+        get_logger().warn(
+            f"Zenodo cache not yet published for this repository."
+        )
+
     # Process each user rule
     for ur in user_rules:
 
@@ -68,35 +74,4 @@ def process_user_rules():
         if not ur.conda_env:
             ur.conda_env = "environment.yml"
 
-    # No need to go any further if this isn't the main build
-    if get_run_type() != "main":
-        return
-
-    # Add some metadata containing the link to the Zenodo cache record
-    # which we can access on the LaTeX side to provide margin links
-    # for figures that depend on cached datasets
-    if cache_zenodo_doi:
-
-        # Add the metadata to the config (accessed in `pdf.py`)
-        zenodo_cache_url = f"https://zenodo.org/record/{cache_zenodo_doi}"
-        labels = {}
-        for figscript in snakemake.workflow.config["dependencies"]:
-            for dep in snakemake.workflow.config["dependencies"][figscript]:
-                if dep in cached_deps:
-                    for label in snakemake.workflow.config["labels"]:
-                        if (
-                            label.endswith("_script")
-                            and snakemake.workflow.config["labels"][label]
-                            == figscript
-                        ):
-                            labels[label[:-6] + "cache"] = zenodo_cache_url
-        snakemake.workflow.config["labels"].update(labels)
-
-    else:
-
-        if cache_sandbox_doi:
-            # There's a Sandbox deposit but no Zenodo deposit; remind
-            # the user to publish the deposit
-            get_logger().warn(
-                f"Zenodo cache not yet published for this repository."
-            )
+    snakemake.workflow.config["cached_deps"] = cached_deps
