@@ -2,23 +2,24 @@ r"""
 Does a first pass through the manuscript to infer the workflow graph.
 
 This does a fast compile of the article, overriding commands like
-``\label``, ``\script``, ``\variable``, and 
+``\label``, ``\script``, ``\variable``, and
 ``\includegraphics`` to instead print their arguments to an XML
 log file, which we use to infer the relationships between input and output
 files. This information is then used to build the workflow graph for the
 main article build step.
 
 """
-from showyourwork import paths, exceptions, zenodo
-from showyourwork.zenodo import Zenodo
-from showyourwork.tex import compile_tex
-import sys
 import json
 import re
-from pathlib import Path
 from collections.abc import MutableMapping
+from pathlib import Path
 from xml.etree.ElementTree import parse as ParseXMLTree
 
+from showyourwork import exceptions, paths, zenodo
+from showyourwork.config import get_upstream_dependencies
+from showyourwork.tex import compile_tex
+from showyourwork.zenodo import Zenodo, get_dataset_urls
+
 
 def flatten_dataset_contents(d, parent_key="", default_path=None):
     """
@@ -205,14 +206,6 @@ def check_figure_format(figure):
                     "Command \marginicon must always come before the figure label."
                 )
 
-    # Check that there is at most one label
-    if len(labels) >= 2:
-        raise exceptions.FigureFormatError(
-            "A figure has multiple labels: `{}`".format(
-                ", ".join(label.text for label in labels)
-            )
-        )
-
     # Check that there is at most one script
     if len(scripts) >= 2:
         raise exceptions.FigureFormatError(
@@ -272,7 +265,7 @@ def get_xml_tree():
 
 def get_json_tree():
     """Builds a dictionary containing mappings between input and output files.
-    
+
     Returns:
         dict: The JSON dependency tree for the article.
     """
@@ -395,26 +388,13 @@ def get_json_tree():
         # Collect user-defined dependencies
         dependencies = config["dependencies"].get(script, [])
 
-        # If any of the dependencies exists in a Zenodo deposit, infer
-        # its URL here so we can add margin links to the PDF
-        datasets = []
-        for doi in config["datasets"]:
-            deposit = Zenodo(doi)
-            url = f"https://{deposit.url}/record/{deposit.deposit_id}"
-            for dep in dependencies:
-                if dep in config["datasets"][doi]["contents"].values():
-                    datasets.append(url)
-                else:
-                    for zip_file in config["datasets"][doi]["zip_files"]:
-                        if (
-                            dep
-                            in config["datasets"][doi]["zip_files"][
-                                zip_file
-                            ].values()
-                        ):
-                            datasets.append(url)
-                            break
-        datasets = list(set(datasets))
+        # Same, but recursing all the way up the graph
+        # (i.e., including dependendencies of dependencies, and so forth)
+        upstream = get_upstream_dependencies(script, config["dependencies"])
+
+        # If any of the upstream dependencies exist in a Zenodo deposit, infer
+        # their URLs so we can add margin links to the PDF
+        datasets = get_dataset_urls(upstream, config["datasets"])
 
         # Format the command by replacing placeholders
         if command is not None:
@@ -432,6 +412,7 @@ def get_json_tree():
             "datasets": datasets,
             "dependencies": dependencies,
             "command": command,
+            "static": static,
         }
 
     # Parse free-floating graphics
@@ -472,6 +453,7 @@ def get_json_tree():
         "datasets": [],
         "dependencies": [],
         "command": None,
+        "static": False,
     }
 
     # Add entries to the tree: static figures
@@ -493,6 +475,7 @@ def get_json_tree():
         "datasets": [],
         "dependencies": [],
         "command": f"cp {srcs} {dest}",
+        "static": True,
     }
 
     # Parse files included using the \input statement;
@@ -538,17 +521,6 @@ if __name__ == "__main__":
         [Path(file).as_posix() for file in config["tree"]["files"]]
     )
 
-    # Gather the figure script & dataset info so we can access it on the TeX side
-    config["labels"] = {}
-    for label, value in config["tree"]["figures"].items():
-        script = value["script"]
-        if script is not None:
-            config["labels"][f"{label}_script"] = script
-        datasets = value["datasets"]
-        # Note: built-in max of 3 datasets will be displayed
-        for dataset, number in zip(datasets, ["One", "Two", "Three"]):
-            config["labels"][f"{label}_dataset{number}"] = dataset
-
     # Save the config file
     with open(config["config_json"], "w") as f:
-        print(json.dumps(config, indent=4), file=f)
\ No newline at end of file
+        print(json.dumps(config, indent=4), file=f)
