@@ -2,16 +2,19 @@
 Main Zenodo interface.
 
 """
-from . import exceptions, paths, git
-from .subproc import parse_request
-from .logging import get_logger
-import requests
-import tarfile
-import os
 import json
-from pathlib import Path
-import subprocess
+import os
 import shutil
+import subprocess
+import tarfile
+from pathlib import Path
+
+import requests
+
+from . import exceptions, git, paths
+from .config import get_run_type
+from .logging import get_logger
+from .subproc import parse_request
 
 try:
     import snakemake
@@ -38,6 +41,49 @@ def require_access_token(method):
     return wrapper
 
 
+def get_dataset_urls(files, datasets):
+    """
+    Given a list of `files`, return all associated Zenodo and/or Zenodo Sandbox
+    URLs.
+
+    This is used to populate the figure margin icons in the article.
+
+    """
+    result = []
+    for doi in datasets:
+        deposit = Zenodo(doi)
+        url = f"https://{deposit.url}/record/{deposit.deposit_id}"
+        for file in files:
+            if file in datasets[doi]["contents"].values():
+                result.append(url)
+            else:
+                for zip_file in datasets[doi]["zip_files"]:
+                    if file in datasets[doi]["zip_files"][zip_file].values():
+                        result.append(url)
+                        break
+    return list(set(result))
+
+
+def get_dataset_dois(files, datasets):
+    """
+    Given a list of `files`, return all associated Zenodo and/or Zenodo Sandbox
+    DOIs.
+
+    """
+    result = []
+    for doi in datasets:
+        deposit = Zenodo(doi)
+        for file in files:
+            if file in datasets[doi]["contents"].values():
+                result.append(doi)
+            else:
+                for zip_file in datasets[doi]["zip_files"]:
+                    if file in datasets[doi]["zip_files"][zip_file].values():
+                        result.append(doi)
+                        break
+    return list(set(result))
+
+
 services = {
     "zenodo": {
         "url": "zenodo.org",
@@ -109,7 +155,14 @@ class Zenodo:
                 raise exceptions.InvalidZenodoDOI(self.doi)
 
             # Check if the user is an owner
-            self.user_is_owner = self.check_if_user_is_owner()
+            try:
+                self.user_is_owner = self.check_if_user_is_owner()
+            except Exception as e:
+                # Fail silently on connection errors
+                logger = get_logger()
+                logger.debug(f"Error accessing the {self.url} API:")
+                logger.debug(str(e))
+                self.user_is_owner = False
 
     def _get_access_token(self, error_if_missing=False):
         """
@@ -255,7 +308,14 @@ class Zenodo:
         # See if we find the deposit
         if r.status_code <= 204:
             if type(r.json()) is list and len(r.json()):
-                logger.info(f"User authentication for {self.doi} is valid.")
+                if get_run_type() == "build":
+                    logger.info(
+                        f"User authentication for {self.doi} is valid."
+                    )
+                else:
+                    logger.debug(
+                        f"User authentication for {self.doi} is valid."
+                    )
                 cache_file_true.touch()
                 return True
             else:
@@ -360,7 +420,9 @@ class Zenodo:
         )
 
     @require_access_token
-    def download_file_from_draft(self, draft, file, rule_name, tarball=False):
+    def download_file_from_draft(
+        self, draft, file, rule_name, tarball=False, dry_run=False
+    ):
         """
         Downloads a file from a Zenodo draft.
 
@@ -400,33 +462,35 @@ class Zenodo:
             ):
 
                 # Download it
-                logger.debug(f"File name and hash both match. Downloading...")
-                url = entry["links"]["download"]
-                progress_bar = (
-                    ["--progress-bar"]
-                    if not snakemake.workflow.config["github_actions"]
-                    else []
-                )
-                try:
-                    res = subprocess.run(
-                        [
-                            "curl",
-                            "-f",
-                            f"{url}?access_token={self.access_token}",
-                            *progress_bar,
-                            "--output",
-                            str(file),
-                        ]
+                logger.debug(f"File name and hash both match.")
+                if not dry_run:
+                    logger.debug("Downloading...")
+                    url = entry["links"]["download"]
+                    progress_bar = (
+                        ["--progress-bar"]
+                        if not snakemake.workflow.config["github_actions"]
+                        else []
                     )
-                except:
-                    raise exceptions.ZenodoDownloadError()
+                    try:
+                        res = subprocess.run(
+                            [
+                                "curl",
+                                "-f",
+                                f"{url}?access_token={self.access_token}",
+                                *progress_bar,
+                                "--output",
+                                str(file),
+                            ]
+                        )
+                    except:
+                        raise exceptions.ZenodoDownloadError()
 
-                # If it's a directory tarball, extract it
-                if tarball:
-                    os.rename(file, f"{file}.tar.gz")
-                    with tarfile.open(f"{file}.tar.gz") as tb:
-                        tb.extractall(file)
-                    Path(f"{file}.tar.gz").unlink()
+                    # If it's a directory tarball, extract it
+                    if tarball:
+                        os.rename(file, f"{file}.tar.gz")
+                        with tarfile.open(f"{file}.tar.gz") as tb:
+                            tb.extractall(file)
+                        Path(f"{file}.tar.gz").unlink()
 
                 return
 
@@ -449,7 +513,7 @@ class Zenodo:
         raise exceptions.FileNotFoundOnZenodo(rule_name)
 
     def download_file_from_record(
-        self, record, file, rule_name, tarball=False
+        self, record, file, rule_name, tarball=False, dry_run=False
     ):
         """
         Downloads a file from a published Zenodo record.
@@ -474,28 +538,31 @@ class Zenodo:
             ):
 
                 # Download it
-                url = entry["links"]["self"]
-                progress_bar = (
-                    ["--progress-bar"]
-                    if not snakemake.workflow.config["github_actions"]
-                    else []
-                )
-                subprocess.run(
-                    [
-                        "curl",
-                        url,
-                        *progress_bar,
-                        "--output",
-                        str(file),
-                    ]
-                )
+                logger.debug(f"File name and hash both match.")
+                if not dry_run:
+                    logger.debug("Downloading...")
+                    url = entry["links"]["self"]
+                    progress_bar = (
+                        ["--progress-bar"]
+                        if not snakemake.workflow.config["github_actions"]
+                        else []
+                    )
+                    subprocess.run(
+                        [
+                            "curl",
+                            url,
+                            *progress_bar,
+                            "--output",
+                            str(file),
+                        ]
+                    )
 
-                # If it's a directory tarball, extract it
-                if tarball:
-                    os.rename(file, f"{file}.tar.gz")
-                    with tarfile.open(f"{file}.tar.gz") as tb:
-                        tb.extractall(file)
-                    Path(f"{file}.tar.gz").unlink()
+                    # If it's a directory tarball, extract it
+                    if tarball:
+                        os.rename(file, f"{file}.tar.gz")
+                        with tarfile.open(f"{file}.tar.gz") as tb:
+                            tb.extractall(file)
+                        Path(f"{file}.tar.gz").unlink()
 
                 return
 
@@ -589,9 +656,10 @@ class Zenodo:
         )
         logger.info(f"Successfully published deposit {self.doi}.")
 
-    def download_file(self, file, rule_name, tarball=False):
+    def download_file(self, file, rule_name, tarball=False, dry_run=False):
         """
         Download a file from the record, deposit or deposit draft.
+
         """
         # Logger
         logger = get_logger()
@@ -632,7 +700,11 @@ class Zenodo:
                         try:
                             draft = r.json()
                             self.download_file_from_draft(
-                                draft, file, rule_name, tarball=tarball
+                                draft,
+                                file,
+                                rule_name,
+                                tarball=tarball,
+                                dry_run=dry_run,
                             )
                         except exceptions.FileNotFoundOnZenodo:
                             exceptions.restore_trace()
@@ -710,7 +782,11 @@ class Zenodo:
                 for record in records[::-1]:
                     try:
                         self.download_file_from_record(
-                            record, file, rule_name, tarball=tarball
+                            record,
+                            file,
+                            rule_name,
+                            tarball=tarball,
+                            dry_run=dry_run,
                         )
                     except exceptions.FileNotFoundOnZenodo:
                         exceptions.restore_trace()
